 package com.biganto.visual.roompark.di.dagger

import com.biganto.visual.androidplayer.data.repository.local.ILocalStore
import com.biganto.visual.roompark.data.local.UserState
import com.biganto.visual.roompark.data.repository.api.IRoomParkApi
import com.biganto.visual.roompark.data.repository.db.IDb
import com.biganto.visual.roompark.data.repository.mapper.fromRaw
import com.biganto.visual.roompark.domain.contract.AuthContract
import com.biganto.visual.roompark.domain.model.AuthInfoModel
import com.biganto.visual.roompark.domain.model.fromEntity
import dagger.Module
import dagger.Provides
import io.reactivex.Completable
import io.reactivex.Observable
import timber.log.Timber
import javax.inject.Inject
import javax.inject.Singleton

 /**
 * Created by Vladislav Bogdashkin on 13.06.2018.
 */

const val USER_CACHE_LIMIT_SIZE = 3
const val USER_CACHE_LIMIT_SECONDS_INACTIVE = 30L

const val TOURS_CACHE_LIMIT_SIZE = 500
const val TOURS_CACHE_LIMIT_SECONDS_INACTIVE = 200L

const val ESTATES_CACHE_LIMIT_SIZE = 100
const val ESTATES_CACHE_LIMIT_SECONDS_INACTIVE = 200L

const val FILES_CACHE_LIMIT_SIZE = 10000
const val FILES_CACHE_LIMIT_SECONDS_INACTIVE = 60L

const val DATABASE_VERSION = 6

 @Module
class AppModule() {
}


 class AuthContractModule @Inject constructor(
    val local:ILocalStore,
    val api:IRoomParkApi,
    val db:IDb
 ):AuthContract{

     init {
         Timber.d("Auth Repository Created")
     }

  override fun signOut(): Completable = local.setRecentUser(null)

  override fun signIn(email: String, password: String): Observable<AuthInfoModel> =
      api.authenticate(email,password)
          .map ( ::fromRaw )
          .flatMap{ db.upsertUser(it)  }
          .doOnNext { local.setRecentUser(it.uuid.toString()) }
          .map(::fromEntity)

  override fun validateAuthState(): Observable<Boolean> = local.recentUser()
      .map { when(it){
          is UserState.NotAuthenticated -> false
          is UserState.Authenticated -> true
          else ->false
      } }
 }

@Module()
class DataModule() {


    @Provides
    @Singleton
    fun provideAuthContract() : AuthContract = AuthContractModule()

/*
    @Provides
    @Singleton
    fun provieApi(@Named("bigantoApi") retorfit:Retrofit): IApi {
        return RetrofitRepository(retorfit)
    }


    @Provides
    @Singleton
    fun provieRoomParkApi(@Named("roomParkApi") retorfit:Retrofit): IRoomParkApi {
        return RoomParkRetrofitRepository(retorfit)
    }


    @Provides
    @Singleton
    fun provideDb(context:Application): IDb {
        return RequeryRepository(getRequeryDataStore(context))
    }

    @Provides
    @Singleton
    fun provideFileModule(context:Application): FilesModule {
        return FilesModule(context)
    }

    @Provides
    @Singleton
    fun providesNetworkListener(context:Application): INetworkMonitor {
        return LiveNetworkMonitor(context)
    }



    @Provides
    @Singleton
    fun getRequeryDataStore(context:Application): KotlinReactiveEntityStore<Persistable> {
        Timber.d("Kotlin store creating..")
        val source = DatabaseSource(context, Models.DEFAULT, "BigantoPerfect", DATABASE_VERSION)

        source.setLoggingEnabled(false)
//        if (  BuildConfig.DEBUG) {
//            // use this in development mode to drop and recreate the tables on every upgrade
//            source.setTableCreationMode(TableCreationMode.DROP_CREATE)
//        }

        val store = KotlinEntityDataStore<Persistable>(source.configuration)
        Timber.d("Kotlin store %s",source)
        return KotlinReactiveEntityStore(store)
//        // override onUpgrade to handle migrating to a new version
//        val configuration = source.configuration
//        return ReactiveSupport.toReactiveStore(
//                EntityDataStore<Persistable>(configuration))
    }

    */
}



//
//@Singleton // <-- YES
//class MyServiceImpl @Inject // <-- YES
//constructor(private val apiService: ApiService)// <-- YES
//    : MyService